/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef TESTPLUGIN1_H
#define TESTPLUGIN1_H
#include <QtGui>

#include "PluginServices.h"
#include "CubePlugin.h"
#include "TabInterface.h"
#include "BladeToolBar.h"

class BladePlugin : public QObject, public cubepluginapi::CubePlugin, cubepluginapi::TabInterface, cubepluginapi::SettingsHandler
{
    Q_OBJECT
    Q_INTERFACES( cubepluginapi::CubePlugin )

#if QT_VERSION >= 0x050000
    Q_PLUGIN_METADATA( IID "BladePlugin" )
#endif

public:
    // CubePlugin implementation
    virtual bool
    cubeOpened( cubepluginapi::PluginServices* service );
    virtual void
    cubeClosed();
    virtual QString
    name() const;
    virtual void
    version( int& major,
             int& minor,
             int& bugfix ) const;
    virtual QString
    getHelpText() const;

    // TabInterface implementation
    virtual QString
    label() const;
    virtual QIcon
    icon() const;
    virtual QWidget*
    widget();
    virtual void
    valuesChanged();
    virtual void
    setActive( bool active );
    virtual QSize
    sizeHint() const;
    // SettingsHandler implementation
    virtual void
    loadGlobalSettings( QSettings& settings );
    virtual void
    saveGlobalSettings( QSettings& settings );
    virtual QString
    settingName();
    
    void
    setGlobalNotificationValue();
    
    bool
    checkForSalsaViewer();
    
    

private slots:
    void
    treeItemIsSelected( cubepluginapi::TreeType  type,
                        cubepluginapi::TreeItem* item );
    
    void
    tabActivated( cubepluginapi::TreeType  type );


    void
    orderHasChanged( const QList<cubepluginapi::DisplayType>& order );


    void
    checkLoop( cubepluginapi::UserAction type );
    
    void
    globalValueChanged( const QString& name );
    
private:

    QWidget*                     widget_;

    cubepluginapi::PluginServices* service;
    
    BladeToolBar * toolBar;
    bool active;
    
    
};


#endif // TESTPLUGIN1_H
