/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef LOAD_FILTERS_WIDGET_H
#define LOAD_FILTERS_WIDGET_H
#include <QtGui>
#include <QLineEdit>
#include <QLabel>
#include <QWidget>
#include <QSpinBox>
#include <QPushButton>
#include <QCheckBox>
#include <QProgressBar>
#include <QVBoxLayout>
#include <QGraphicsScene>
#include <QGraphicsView>
#include "Trace.h"

class TraceLoadEnterFilter;
class TraceLoadLeaveFilter;

class LoadFiltersWidget : public QWidget
{
    Q_OBJECT
    
private:
    
    Trace * trace;
    TraceLoadEnterFilter * trace_load_enter_filter;
    TraceLoadLeaveFilter * trace_load_leave_filter;
    
    
    QPushButton * full_load;
    QPushButton * preview_load;
    QPushButton * custom_load;
    QPushButton * load_trace;
    QLineEdit * get_start_timestamp_input;
    QLineEdit * get_end_timestamp_input;
    QSpinBox * call_stack_input;
    QLineEdit * region_name_input;
    QLineEdit * region_paradigm_input;
    QLineEdit * region_role_input;
    QCheckBox * resolution_check;
    QCheckBox * processing_check;
    
    
public slots:
    void
    startLoadingTrace();
    
    void
    setFullLoad();
    
    void
    setPreviewLoad();
    
    void
    setCustomLoad();
    
    
    
signals:
    void
    loadTrace();
    
    
    
public:
    LoadFiltersWidget(Trace * _trace,  QWidget * _parent = NULL);
    ~LoadFiltersWidget();
 
    TraceLoadEnterFilter*
    get_trace_load_enter_filter();
    
    TraceLoadLeaveFilter*
    get_trace_load_leave_filter();  
    
    
    bool
    do_grouping() const;
    
};

#endif // TESTPLUGIN1_H
