/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef METRIC_GRAPHICS_VIEW_H
#define METRIC_GRAPHICS_VIEW_H


#include <QGraphicsView>
#include <QGraphicsRectItem>
#include "MetricLine.h"

#include "Trace.h"
extern Trace * trace;

class MetricGraphicsView : public QGraphicsView  
{
    Q_OBJECT
    
private:

    QGraphicsRectItem * zoom;
    QGraphicsLineItem * cursor;
    QPoint start_position;
    QPoint end_position;
    QRect zoom_rect;
    
    QPoint cursorPosition;
    
    QPointF zoom_window;
    bool zooming;
    bool pressed;
    int x_treshold;
    
    
    MetricLine *metric_line;
    
    
    QRect 
    calculateRect( const QPoint&, const QPoint&);
    
    QPointF
    calculateZoomF( );
    

    
public:

    MetricGraphicsView ( QGraphicsScene* scene, MetricLine * );

    void
    updateScene();

    QPointF&
    getZoomF();
     
    QPointF
    getCursorPosition();
    
    void
    setCursorPosition(QPointF& );
    
    
    void mousePressEvent(QMouseEvent * event);


    void mouseMoveEvent(QMouseEvent * event);


    void mouseReleaseEvent(QMouseEvent * event);
    
    
    void wheelEvent ( QWheelEvent * event )
    {}
    
    void keyPressEvent ( QKeyEvent * event ) 
    {}    
    
    void
    showToolTip( QString );
     
     
signals:
    void
    zoomChanged();
    
    void
    zoomingSelection();
    
    void
    cursorMoved(QPointF);
    
    
};


#endif // TESTPLUGIN1_H
