/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef METRIC_TRACE_WIDGET_H
#define METRIC_TRACE_WIDGET_H
#include <QtGui>
#include <QLabel>
#include <QWidget>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QLineEdit>

#include "MetricGraphicsView.h"

#include "Trace.h"
#include "CallPathFilter.h"
#include "FlatFilter.h"
#include "SelectionFilter.h"
#include "ZoomFilter.h"
#include "ResolutionFilter.h"
#include "LocationFilter.h"
#include "ColorMap.h"
#include "MetricLine.h"

#include "GraphicsView.h"

#include "PluginServices.h"


extern cubepluginapi::PluginServices* blade_service;
extern Trace * trace;
extern ZoomFilter * z_filter;

extern SelectionFilter * s_filter;

extern TraceEventFilter  * cf_filter;

class MetricTraceWidget : public QWidget
{
    Q_OBJECT
private:
    QGraphicsScene * scene;
    MetricGraphicsView * view;

    QGraphicsRectItem * zoom;
    QRect zoom_rect;
    
    QPoint cursorPosition;
    QGraphicsLineItem * cursor;
    QGraphicsSimpleTextItem * min_label;
    QGraphicsSimpleTextItem * max_label;
    QGraphicsSimpleTextItem * metric_label;
    
    
    std::vector<TraceEvent*> events;
    uint64_t margin;
    
    QString metric_expression;
    MetricLine * metric_line;
    
    
    
public slots:
    void 
    changeZoom();
    
    void
    zoomingTip();
    
    void
    updateScene(); 
    
    void
    updateCursor(QPointF);
    
    void
    addCounterLine();
    
    
    
public:
    MetricTraceWidget( QWidget * parent = NULL);
    ~MetricTraceWidget();
    
    void 
    resizeEvent(QResizeEvent *event);
    
    QPointF
    getCursor();
    
    void
    setCursor( QPointF&);
    
    
    void
    set_trace_view( const std::vector<TraceEvent*> & );
    
    
signals:
    void
    cursorMoved(QPointF);

    
};

#endif // TESTPLUGIN1_H
