/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef MPI_COLLECTIVE_TRACE_EVENT_GROUP_COLLECTION_H
#define MPI_COLLECTIVE_TRACE_EVENT_GROUP_COLLECTION_H

#include <vector>

#include "TraceEventGroupsCollection.h"
#include "MpiCollectiveGroup.h"

class MpiCollectiveGroupsCollection : public TraceEventGroupCollection<MpiCollectiveGroup>
{
protected:
   std::vector<uint64_t> indexes;
    
public:
    MpiCollectiveGroupsCollection(uint64_t _num_locations)
    {
        indexes.resize(_num_locations);
    }
 
    inline
    virtual
    bool
    find_and_delete( TraceEvent * _e)
    {
	uint64_t _location = _e->get_location();
	for (std::vector<MpiCollectiveGroup*>::iterator iter=groups.begin(); iter != groups.end(); ++iter)
	{
	    MpiCollectiveGroup* _g = *iter;
	    _g->delete_event(_e);
	}
	
    }
    
    inline 
    virtual
    MpiCollectiveGroup*  add_event( TraceEvent * _e, uint64_t _l)
    {
        MpiCollectiveGroup * mg = NULL;
        if (indexes[_l]>= groups.size())
        {
            groups.resize( groups.size()+1); 
            groups[groups.size()-1 ] = new MpiCollectiveGroup();
            if (groups.size()>1)
            {
                groups[groups.size()-1 ]->set_prev_group( groups[groups.size()-2 ] );
                groups[groups.size()-2 ]->set_next_group( groups[groups.size()-1 ]);
            }
            groups[groups.size()-1]->set_num_locations( indexes.size());
        }
        mg = (groups[indexes[_l]  ]);
        indexes[_l]++;
        mg->add_event(_e);
        return mg;
    }
    
};






#endif // MPI_COLLECTIVE_TRACE_EVENT_GROUP_COLLECTION_H
