/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef MPI_P2P_TRACE_EVENT_GROUP_COLLECTION_H
#define MPI_P2P_TRACE_EVENT_GROUP_COLLECTION_H

#include <vector>
#include <map>

#include "Service.h"
#include "TraceEventGroupsCollection.h"
#include "MpiP2PGroup.h"
#include "SendP2PTraceEvent.h"
#include "RecvP2PTraceEvent.h"
#include "ISendP2PTraceEvent.h"
#include "IRecvP2PTraceEvent.h"
#include "WaitTraceEvent.h"
#include "WaitallTraceEvent.h"


class MpiP2PGroupsCollection : public TraceEventGroupCollection<MpiP2PGroup>
{
protected:
    std::vector<std::map<void*, SendP2PTraceEvent*> > send_events;
    std::vector<std::map<void*, ISendP2PTraceEvent*> > isend_events;

    std::vector<std::map<void*, RecvP2PTraceEvent*> > recv_events;
    std::vector<std::map<void*, IRecvP2PTraceEvent*> > irecv_events;
    
    
    std::vector<std::map<void*, SendP2PTraceEvent*> > matched_send_events;
    std::vector<std::map<void*, ISendP2PTraceEvent*> >  matched_isend_events;

    std::vector<std::map<void*, RecvP2PTraceEvent*> >  matched_recv_events;
    std::vector<std::map<void*, IRecvP2PTraceEvent*> > matched_irecv_events;
    

    std::vector<std::map<void*, ISendP2PTraceEvent*> > incomplete_isend_events;
    std::vector<std::map<void*, IRecvP2PTraceEvent*> > incomplete_irecv_events;   
     
    double progress_step;
    
    
   
public:
    MpiP2PGroupsCollection(uint64_t _num_locations)
    {
        send_events.resize(_num_locations);
        recv_events.resize(_num_locations);
        isend_events.resize(_num_locations);
        irecv_events.resize(_num_locations);
        
        matched_send_events.resize(_num_locations);
        matched_recv_events.resize(_num_locations);
        matched_isend_events.resize(_num_locations);
        matched_irecv_events.resize(_num_locations);
            
        incomplete_isend_events.resize(_num_locations);
        incomplete_irecv_events.resize(_num_locations);
        
        progress_step = 0;
    }
 
 
    inline
    virtual
    bool
    find_and_delete( TraceEvent * _e)
    {
	SendP2PTraceEvent * _as_send = dynamic_cast<SendP2PTraceEvent*>(_e);        
        RecvP2PTraceEvent * _as_recv = dynamic_cast<RecvP2PTraceEvent*>(_e);
        ISendP2PTraceEvent * _as_isend = dynamic_cast<ISendP2PTraceEvent*>(_e);    
	IRecvP2PTraceEvent * _as_irecv = dynamic_cast<IRecvP2PTraceEvent*>(_e);
	
	uint64_t _location = bladeservices::master_id(_e->get_location());
	
	if (_as_send!= NULL)
        {
	    send_events[ _location ].erase( _as_send );
	    matched_send_events[ _location ].erase( _as_send  );
        }
        else
	if (_as_isend!= NULL)
        {
	    isend_events[ _location ].erase( _as_isend );
	    matched_isend_events[ _location ].erase( _as_isend );
	    incomplete_isend_events[ _location ].erase( _as_isend );
        }
	else
	if (_as_recv!= NULL)
        {
	    recv_events[ _location ].erase( _as_recv  );
	    matched_recv_events[ _location ].erase( _as_recv );
        }
        else
	if (_as_irecv!= NULL)
        {
	    irecv_events[ _location ].erase(  _as_irecv  );
	    matched_irecv_events[ _location ].erase( _as_irecv );
	    incomplete_irecv_events[ _location ].erase( _as_irecv );
        }
    }
    
 
 
    inline 
    virtual
    void 
    add_incomplete_event( TraceEvent * _e, uint64_t _l)
    {
        IRecvP2PTraceEvent * _as_irecv = dynamic_cast<IRecvP2PTraceEvent*>(_e);
        if (_as_irecv  != NULL)
            incomplete_irecv_events[bladeservices::master_id(_l)].insert( std::pair<void*,  IRecvP2PTraceEvent * >(_as_irecv,_as_irecv) );
        ISendP2PTraceEvent * _as_isend = dynamic_cast<ISendP2PTraceEvent*>(_e);
        if (_as_isend  != NULL)
            incomplete_isend_events[bladeservices::master_id(_l)].insert( std::pair<void*,  ISendP2PTraceEvent * >(_as_isend,_as_isend) );
    }
    
    
    
    inline 
    virtual
    MpiP2PGroup* 
    add_event( TraceEvent * _e, uint64_t _l)
    {
        SendP2PTraceEvent * _as_send = dynamic_cast<SendP2PTraceEvent*>(_e);        
        RecvP2PTraceEvent * _as_recv = dynamic_cast<RecvP2PTraceEvent*>(_e);
        ISendP2PTraceEvent * _as_isend = dynamic_cast<ISendP2PTraceEvent*>(_e);        
         
        if (_as_send != NULL)
            send_events[bladeservices::master_id(_l)].insert( std::pair<void*,  SendP2PTraceEvent * >(_as_send,_as_send) );
        if (_as_recv != NULL)
            recv_events[bladeservices::master_id(_l)].insert( std::pair<void*,  RecvP2PTraceEvent * >(_as_recv,_as_recv) );
        if (_as_isend != NULL)
            isend_events[bladeservices::master_id(_l)].insert( std::pair<void*,  ISendP2PTraceEvent * >(_as_isend,_as_isend) );
        return NULL;     
    }
  
  
  
    inline 
    virtual
    void 
    complete_irecv_event( TraceEvent * _e, uint64_t _l)
    {
        WaitTraceEvent * _as_wait = dynamic_cast<WaitTraceEvent*>(_e);
        WaitallTraceEvent * _as_waitall = dynamic_cast<WaitallTraceEvent*>(_e);
        if (_as_wait != NULL)
        {
            std::map<void*, IRecvP2PTraceEvent*>::iterator _l_iter = incomplete_irecv_events[bladeservices::master_id(_l)].begin();   
            while ( 
		_l_iter != incomplete_irecv_events [bladeservices::master_id(_l)].end() 
		&& 		
                !(_as_wait->match_request( _l_iter->second)) 
            ){ ++_l_iter;}
            if (_l_iter != incomplete_irecv_events [bladeservices::master_id(_l)].end()) 
            {
                IRecvP2PTraceEvent* _as_irecv = _l_iter->second;
                
                _as_irecv->init( _as_wait->get_tag(), _as_wait->get_p2p_partner(), _as_wait->get_communicator(), _as_wait->get_request() );
                (_as_irecv)->set_waiting_event( _as_wait );
                irecv_events[bladeservices::master_id(_l)].insert( std::pair<void*,  IRecvP2PTraceEvent * >(_as_irecv,_as_irecv) );
                incomplete_irecv_events[bladeservices::master_id(_l)].erase( _l_iter);
                _as_wait->set_completed_req_events(_as_irecv);
            }   
        }
        if (_as_waitall!= NULL)
        {
           std::map<void*, IRecvP2PTraceEvent*>::iterator _l_iter = incomplete_irecv_events[bladeservices::master_id(_l)].begin();   
            while ( 
		_l_iter != incomplete_irecv_events [bladeservices::master_id(_l)].end() 
		&& 
                !(_as_waitall->match_request( _l_iter->second)) 
            ){ ++_l_iter;}
            if (_l_iter != incomplete_irecv_events [bladeservices::master_id(_l)].end()) 
            {
                IRecvP2PTraceEvent* _as_irecv = (_l_iter->second);
                
                _as_irecv->init( _as_waitall->get_tag(), _as_waitall->get_p2p_partner(), _as_waitall->get_communicator(), _as_waitall->get_request() );
                (_as_irecv)->set_waiting_event( _as_waitall );
                irecv_events[bladeservices::master_id(_l)].insert( std::pair<void*,  IRecvP2PTraceEvent * >(_as_irecv,_as_irecv) );
                incomplete_irecv_events[bladeservices::master_id(_l)].erase( _l_iter);
                _as_waitall->add_completed_req_events(_as_irecv);
            }
        }
    }
    
 
    inline 
    virtual
    void 
    complete_isend_event( TraceEvent * _e, uint64_t _l)
    {
        WaitTraceEvent * _as_wait = dynamic_cast<WaitTraceEvent*>(_e);
        WaitallTraceEvent * _as_waitall = dynamic_cast<WaitallTraceEvent*>(_e);
        if (_as_wait != NULL)
        {
            std::map<void*, ISendP2PTraceEvent*>::iterator _l_iter = incomplete_isend_events[bladeservices::master_id(_l)].begin();   
            while ( 
		_l_iter != incomplete_isend_events [bladeservices::master_id(_l)].end() 
		&& 
                !(_as_wait->match_request( _l_iter->second)) 
            ){ ++_l_iter;}
            if (_l_iter != incomplete_isend_events [bladeservices::master_id(_l)].end()) 
            {
                ISendP2PTraceEvent* _as_isend = (_l_iter->second);
                (_as_isend)->set_waiting_event( _as_wait );
                isend_events[bladeservices::master_id(_l)].insert( std::pair<void*,  ISendP2PTraceEvent * >(_as_isend,_as_isend) );
                incomplete_isend_events[bladeservices::master_id(_l)].erase( _l_iter);
                _as_wait->set_completed_req_events(_as_isend);
            }   
        }
        if (_as_waitall!= NULL)
        {
            std::map<void*, ISendP2PTraceEvent*>::iterator _l_iter = incomplete_isend_events[bladeservices::master_id(_l)].begin();   
            while ( 
		_l_iter != incomplete_isend_events [bladeservices::master_id(_l)].end() 
		&& 
                !(_as_waitall->match_request( _l_iter->second)) 
            ){ ++_l_iter;}
            if (_l_iter != incomplete_isend_events [bladeservices::master_id(_l)].end()) 
            {
                ISendP2PTraceEvent* _as_isend = (_l_iter->second);
                (_as_isend)->set_waiting_event( _as_waitall );
                isend_events[bladeservices::master_id(_l)].insert( std::pair<void*,  ISendP2PTraceEvent * >(_as_isend,_as_isend) );
                incomplete_isend_events[bladeservices::master_id(_l)].erase( _l_iter);
                _as_waitall->add_completed_req_events(_as_isend);
            }   
        }
    }    
    
    inline 
    virtual
    void 
    match_events( )
    {
       progress_message = "Matching synchroneous P2P events...";
       for (size_t si=0; si<send_events.size(); ++si )
        {
            std::map<void*, SendP2PTraceEvent*>::iterator _s_iter = send_events[si].begin();
            while(
                 _s_iter !=  send_events[si].end())
            {
                SendP2PTraceEvent* _as_send = _s_iter->second;
                    
                std::map<void*, IRecvP2PTraceEvent*>::iterator _ir_iter =  irecv_events[_as_send->get_p2p_partner()].begin();   
                while ( 
                    _ir_iter != irecv_events[_as_send->get_p2p_partner()].end() 
                    &&
                    !(_as_send->match(  (P2PTraceEvent*)(_ir_iter->second)) 
                    ) 
                ){ 
                    ++_ir_iter;
                }
                if (_ir_iter != irecv_events [_as_send->get_p2p_partner()].end()) 
                {
                    IRecvP2PTraceEvent* _as_irecv = (_ir_iter->second);

                    (_as_irecv)->set_communication_partner( _as_send );
                    (_as_send)->set_communication_partner( _as_irecv );
                        
                    matched_irecv_events[_as_send->get_p2p_partner()].insert( std::pair<void*,  IRecvP2PTraceEvent * >(_as_irecv,_as_irecv) );
                    irecv_events[_as_send->get_p2p_partner()].erase( _ir_iter);
                    
                    matched_send_events[si].insert( std::pair<void*,  SendP2PTraceEvent * >(_as_send,_as_send) );    
                    std::map<void*, SendP2PTraceEvent*>::iterator _tmp = ++_s_iter;
                    send_events[si].erase( --_s_iter);
                    _s_iter = _tmp;
                    
                    continue;
                }
                std::map<void*, RecvP2PTraceEvent*>::iterator _r_iter =  recv_events[_as_send->get_p2p_partner()].begin();   
                while ( 
                    _r_iter != recv_events[_as_send->get_p2p_partner()].end() 
                    &&
                    !(_as_send->match(  (P2PTraceEvent*)(_r_iter->second))) 
                ){ ++_r_iter;}
                if (_r_iter != recv_events [_as_send->get_p2p_partner()].end()) 
                {
                    RecvP2PTraceEvent* _as_recv = (_r_iter->second);

                    (_as_recv)->set_communication_partner( _as_send );
                    (_as_send)->set_communication_partner( _as_recv );
                        
                    matched_recv_events[_as_send->get_p2p_partner()].insert( std::pair<void*,  RecvP2PTraceEvent * >(_as_recv,_as_recv) );
                    recv_events[_as_send->get_p2p_partner()].erase( _r_iter);
                    
                    matched_send_events[si].insert( std::pair<void*,  SendP2PTraceEvent * >(_as_send,_as_send) );    
                    std::map<void*, SendP2PTraceEvent*>::iterator _tmp = ++_s_iter;
                    send_events[si].erase( --_s_iter);
                    _s_iter = _tmp;
                    
                    continue;
                }   
            ++_s_iter;
            }
            
            progress_step = (double)si/(double ) (send_events.size());
            
        } 
        progress_message = "Matching asynchroneous P2P events...";
        for (size_t isi=0; isi<isend_events.size(); ++isi )
        {
            std::map<void*, ISendP2PTraceEvent*>::iterator _is_iter = isend_events[isi].begin();
            
            while(
                 _is_iter !=  isend_events[isi].end())
            {
                ISendP2PTraceEvent* _as_isend = _is_iter->second;
                    
                std::map<void*, IRecvP2PTraceEvent*>::iterator _ir_iter =  irecv_events[_as_isend->get_p2p_partner()].begin();   
                while ( 
                    _ir_iter != irecv_events[_as_isend->get_p2p_partner()].end() 
                    &&
                    !(_as_isend->match( (P2PTraceEvent*)(_ir_iter->second))) 
                ){ ++_ir_iter;}
                if (_ir_iter != irecv_events [_as_isend->get_p2p_partner()].end()) 
                {
                    IRecvP2PTraceEvent* _as_irecv = (_ir_iter->second);

                    (_as_irecv)->set_communication_partner( _as_isend );
                    (_as_isend)->set_communication_partner( _as_irecv );
                        
                    matched_irecv_events[_as_isend->get_p2p_partner()].insert( std::pair<void*,  IRecvP2PTraceEvent * >(_as_irecv,_as_irecv) );
                    irecv_events[_as_isend->get_p2p_partner()].erase( _ir_iter);
                    
                    matched_isend_events[isi].insert( std::pair<void*,  ISendP2PTraceEvent * >(_as_isend,_as_isend) ); 
                    std::map<void*, ISendP2PTraceEvent*>::iterator _tmp = ++_is_iter;
                    isend_events[isi].erase( --_is_iter);
                    _is_iter = _tmp;
                    continue;
                }
                std::map<void*, RecvP2PTraceEvent*>::iterator _r_iter =  recv_events[_as_isend->get_p2p_partner()].begin();   
                while ( 
                    
                    _r_iter != recv_events[_as_isend->get_p2p_partner()].end() 
                    &&
                    !(_as_isend->match(  (P2PTraceEvent*)(_r_iter->second))) 
                ){ ++_r_iter;}
                if (_r_iter != recv_events [_as_isend->get_p2p_partner()].end()) 
                {
                    RecvP2PTraceEvent* _as_recv = (_r_iter->second);

                    (_as_recv)->set_communication_partner( _as_isend );
                    (_as_isend)->set_communication_partner( _as_recv );
                        
                    matched_recv_events[_as_isend->get_p2p_partner()].insert( std::pair<void*,  RecvP2PTraceEvent * >(_as_recv,_as_recv) );
                    recv_events[_as_isend->get_p2p_partner()].erase( _r_iter);
                    
                    matched_isend_events[isi].insert( std::pair<void*,  ISendP2PTraceEvent * >(_as_isend,_as_isend) ); 
                    std::map<void*, ISendP2PTraceEvent*>::iterator _tmp = ++_is_iter;
                    isend_events[isi].erase( --_is_iter);
                    _is_iter = _tmp;
                    continue;
                }   
                ++_is_iter;
            }
             progress_step = (double)(isi ) /(double ) (isend_events.size());
        } 
        
        
        
    }
    
    inline 
    virtual
    void 
    group_events( )
    {
        std::list<P2PTraceEvent*> _events;
        progress_message = "Collecting all P2P events...";
        for (size_t isi=0; isi<isend_events.size(); ++isi )
        {
            std::map<void*,  SendP2PTraceEvent*>::iterator _s_iter =  matched_send_events[ isi ].begin();   
            while ( _s_iter !=  matched_send_events[ isi ].end())
            {
                _events.push_back( _s_iter->second ); 
                ++_s_iter;
            }
            std::map<void*, ISendP2PTraceEvent*>::iterator _is_iter =  matched_isend_events[ isi ].begin();   
            while ( _is_iter !=  matched_isend_events[ isi ].end())
            {
                _events.push_back( _is_iter->second ); 
                ++_is_iter;
            }
            std::map< void*,RecvP2PTraceEvent*>::iterator _r_iter =  matched_recv_events[ isi ].begin();   
            while ( _r_iter !=  matched_recv_events[ isi ].end())
            {
                _events.push_back( _r_iter->second ); 
                ++_r_iter;
            }
            std::map<void*, IRecvP2PTraceEvent*>::iterator _ir_iter =  matched_irecv_events[ isi ].begin();   
            while ( _ir_iter !=  matched_irecv_events[ isi ].end())
            {
                _events.push_back( _ir_iter->second ); 
                ++_ir_iter;
            }
            progress_step = (double)isi/(double)(isend_events.size());
        } 
        
        int _initial_size = _events.size();
        progress_message = "Grouping P2P events...";
	size_t index = 0;
        std::list<P2PTraceEvent*>::iterator _iter = _events.begin();
        while( _iter != _events.end())
        {
            P2PTraceEvent * _e = *_iter;
	    if ( _e->get_events_group() != NULL)
	    {
		++_iter;
                continue;
	    }
            MpiP2PGroup * mg = new MpiP2PGroup();
                
            if (add_to_group( mg, _e ))
	    {
		groups.push_back( mg );
		if (groups.size()>1)
		{
		    groups[groups.size()-1 ]->set_prev_group( groups[groups.size()-2 ] );
		    groups[groups.size()-2 ]->set_next_group( groups[groups.size()-1 ]);
		}            
	    }
	    else
	    {
		delete mg;
	    }

	progress_step = (double)(index)/(double)_events.size();
	index++;
	++_iter;
        }
   
    }
    
    
    inline 
    virtual
    bool 
    add_to_group( MpiP2PGroup* mg, P2PTraceEvent * _e)
    {
        if (_e->get_events_group() != NULL)
            return false;
        std::list<P2PTraceEvent*> _list_of_events;
        _list_of_events.push_back( _e );
        _e->set_events_group(mg);
        mg->add_event(_e);
        
        std::list<P2PTraceEvent*>::iterator iter = _list_of_events.begin();
        while( iter !=  _list_of_events.end())
        {
                P2PTraceEvent * ___e = *iter;
        
                SendP2PTraceEvent * _as_send = dynamic_cast<SendP2PTraceEvent*>(___e);        
                RecvP2PTraceEvent * _as_recv = dynamic_cast<RecvP2PTraceEvent*>(___e);
                ISendP2PTraceEvent * _as_isend = dynamic_cast<ISendP2PTraceEvent*>(___e);        
                IRecvP2PTraceEvent * _as_irecv = dynamic_cast<IRecvP2PTraceEvent*>(___e);        
                WaitTraceEvent * _as_wait = dynamic_cast<WaitTraceEvent*>(___e); 
                WaitallTraceEvent * _as_waitall = dynamic_cast<WaitallTraceEvent*>(___e);  
                
                
                 P2PTraceEvent * ____e = NULL;
                if (_as_send != NULL)
                {
                    ____e = _as_send->get_communication_partner();
                    if (____e->get_events_group() == NULL)
                    {
                        ____e->set_events_group(mg);
                        mg->add_event(____e);
                        _list_of_events.push_back( ____e );
                    }
                }
                if (_as_isend != NULL)
                {
                     ____e =  _as_isend->get_communication_partner() ;
                     if (____e->get_events_group() == NULL)
                      {
                        ____e->set_events_group(mg);
                        mg->add_event(____e);
                        _list_of_events.push_back( ____e );
                      }
                       ____e =  _as_isend->get_waiting_event() ;
                     if (____e->get_events_group() == NULL)
                      {
                        ____e->set_events_group(mg);
                        mg->add_event(____e);
                        _list_of_events.push_back( ____e );
                      }
                }
                if (_as_recv != NULL)
                {
                    ____e = _as_recv->get_communication_partner();
                    if (____e->get_events_group() == NULL)
                    {
                        ____e->set_events_group(mg);
                        mg->add_event(____e);
                        _list_of_events.push_back( ____e );
                    }
                }
                if (_as_irecv != NULL)
                {
                    ____e =  _as_irecv->get_communication_partner() ;
                     if (____e->get_events_group() == NULL)
                      {
                        ____e->set_events_group(mg);
                        mg->add_event(____e);
                        _list_of_events.push_back( ____e );
                      }
                       ____e =  _as_irecv->get_waiting_event() ;
                     if (____e->get_events_group() == NULL)
                      {
                        ____e->set_events_group(mg);
                        mg->add_event(____e);
                        _list_of_events.push_back( ____e );
                      }
                }        
                if (_as_waitall != NULL)
                {
                    std::vector<RequestP2PTraceEvent*>& _sub_events = _as_waitall->get_completed_req_events( );
                    for ( std::vector<RequestP2PTraceEvent*>::iterator iter = _sub_events.begin(); iter != _sub_events.end(); ++iter)
                    {
                        ____e =  *iter;
                        if (____e->get_events_group() == NULL)
                        {
                            ____e->set_events_group(mg);
                            mg->add_event(____e);
                            _list_of_events.push_back( ____e );
                        }
                    }
                }        
                ++iter;
        }
        return true;
    }
    
    
    inline
    virtual
    double
    get_progress(bladeservices::TraceLoadingProgressStep _progress_step)
    {
        switch (_progress_step)
        {
            case bladeservices::P2PMatching:
            case bladeservices::P2PGrouping:
            default:
                return progress_step;
        }
    }
    
 
 
 
};






#endif // MPI_P2P_TRACE_EVENT_GROUP_COLLECTION_H
