/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef OMP_PARALLEL_TRACE_EVENT_GROUP_COLLECTION_H
#define OMP_PARALLEL_TRACE_EVENT_GROUP_COLLECTION_H

#include <vector>
#include <list>

#include "Service.h"
#include "TraceEventGroupsCollection.h"
#include "OmpParallelGroup.h"
#include "TraceEvent.h"


class OmpParallelGroupsCollection : public TraceEventGroupCollection<OmpParallelGroup>
{
protected:
    std::vector<bool> active;
    std::vector<std::vector<OmpParallelGroup*> > omp_groups;
public:
    OmpParallelGroupsCollection(uint64_t num_locations)
    {
        active.resize( num_locations);
        omp_groups.resize( num_locations );
        for ( size_t i = 0; i< active.size(); ++i)
            active[i] = false;
        
    }
 
    inline
    virtual
    bool
    find_and_delete( TraceEvent * _e)
    {
	uint64_t _location =  bladeservices::master_id( _e->get_location() );
	for (std::vector<OmpParallelGroup*>::iterator iter=omp_groups[_location].begin(); iter != omp_groups[_location].end(); ++iter)
	{
	    OmpParallelGroup* _g = *iter;
	    _g->delete_event(_e);
	}
	
    }
 
    inline
    void
    set_active(uint64_t l, bool _b)
    {
        active[l] = _b;
        if (active[l])
        {
            OmpParallelGroup * _g = new OmpParallelGroup();
            groups.push_back(_g);
            omp_groups[l].push_back(_g);
            if (omp_groups[l].size()>1)
            {
                omp_groups[l][omp_groups[l].size()-1 ]->set_prev_group( omp_groups[l][omp_groups[l].size()-2 ] );
                omp_groups[l][omp_groups[l].size()-2 ]->set_next_group( omp_groups[l][omp_groups[l].size()-1 ]);
            }
        }
    }
    
    
    
    inline 
    virtual
    OmpParallelGroup* 
    add_event( TraceEvent * _e, uint64_t _l )
    {
        if (!active[_l])
            return NULL;
        OmpParallelGroup * mg = omp_groups[_l].back();
        
        mg->add_event(_e);
        return mg;
       
    }
};






#endif // MPI_COLLECTIVE_TRACE_EVENT_GROUP_COLLECTION_H
