/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef OVERVIEW_GRAPHICS_VIEW_H
#define OVERVIEW_GRAPHICS_VIEW_H


#include <QGraphicsView>
#include <QGraphicsRectItem>
#include "ZoomFilter.h"
#include "Trace.h"



class OverviewGraphicsView : public QGraphicsView  
{
    Q_OBJECT
    
private:

    ZoomFilter * z_filter; // for update Widget
    Trace * trace;
    QGraphicsRectItem * zoom;
    QPoint start_position;
    QPoint end_position;
    QRect zoom_rect;
    
    QPointF zoom_window;
    bool zooming;
    
    QRect 
    calculateRect( const QPoint&, const QPoint&);
    
    QPointF
    calculateZoomF( );
    
    
public:

    OverviewGraphicsView ( QGraphicsScene* scene, ZoomFilter * _z_o, Trace * _trace);


    QPointF&
    getZoomF();
     
    void
    updateScene();   
    
    void mousePressEvent(QMouseEvent * event);


    void mouseMoveEvent(QMouseEvent * event);


    void mouseReleaseEvent(QMouseEvent * event);
    
    
    void wheelEvent ( QWheelEvent * event )
    {}
    
    void keyPressEvent ( QKeyEvent * event ) 
    {}    
    
    void
    showToolTip( QString );
    
public slots:
    void
    changeZoom(QPointF);
 
    
signals:
    void
    zoomChanged();
    
    void
    zoomingSelection();
    
    
};


#endif // TESTPLUGIN1_H
