/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef OVERVIEW_TRACE_WIDGET_H
#define OVERVIEW_TRACE_WIDGET_H
#include <QtGui>
#include <QLabel>
#include <QWidget>
#include <QGraphicsView>
#include <QGraphicsScene>

#include "Trace.h"
#include "CallPathFilter.h"
#include "ZoomFilter.h"
#include "ResolutionFilter.h"
#include "FlatFilter.h"
#include "SelectionFilter.h"
#include "ColorMap.h"

#include "OverviewGraphicsView.h"

#include "PluginServices.h"

extern cubepluginapi::PluginServices* blade_service;

class OverviewTraceWidget : public QWidget
{
    Q_OBJECT
private:
    TraceEventFilter ** cf_filter;
    SelectionFilter * s_filter;
    ZoomFilter * z_filter;
    ZoomFilter * z_filters;
    ResolutionFilter * r_filter;
    LocationFilter * l_filter;
    
    
    uint64_t location;
    Trace * trace;
    QGraphicsScene * scene;
    OverviewGraphicsView * view;

    int32_t calltree_height ;
    int32_t calltree_height_step;
    
    std::vector<TraceEvent * > trace_view;

    std::vector< std::vector< QGraphicsRectItem * > > reduced_trace_view; // for every level -> own bars
    std::vector< std::vector<  QGraphicsRectItem * > > reduced_levels_trace_view; // for every level -> own bars
    
    
    ColorPalette * coloring;
   
    void
    recalculateCallTreeStep();
    
    void
    createReducedTraceView();
    
public slots:
    void 
    changeZoom();
    
    void
    zoomingTip();
    
    void
    updateSceneFull(); 
    
    void
    updateScene(bool full=false); 
    
    void
    setLocation( uint64_t );
  
public:
    OverviewTraceWidget(uint64_t, Trace * _trace, TraceEventFilter **_cf, SelectionFilter * _s, ZoomFilter * _z, ZoomFilter * _z_o,  QWidget * parent = NULL);
    ~OverviewTraceWidget();
    
    void 
    resizeEvent(QResizeEvent *event);
    
    
};

#endif // TESTPLUGIN1_H
