/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef P2P_TRACE_EVENT_H
#define P2P_TRACE_EVENT_H

#include <string>
#include <vector>
#include <inttypes.h>
#include <otf2/otf2.h>
#include"TraceEvent.h"




class P2PTraceEvent : public TraceEvent 
{
protected:
    uint64_t tag;
    uint64_t communicator;
    uint64_t p2p_partner;
    
    P2PTraceEvent * com_partner;
    
    
public:

    P2PTraceEvent( uint64_t _location): TraceEvent( _location) 
    {
        com_partner = NULL;
    };
    virtual
    ~P2PTraceEvent() {};
    
    
    inline
    void
    init( uint64_t _t, uint64_t _p2p_partner, uint64_t _c)
    {
        tag = _t;
        p2p_partner = _p2p_partner;
        communicator = _c;
    }
    
    inline 
    bool
    match( uint64_t _t, uint64_t _p2p_partner, uint64_t _c) const
    {
         return ( _t == tag && _p2p_partner == p2p_partner && _c == communicator);
    }
    
    inline 
    bool
    match( P2PTraceEvent * _p ) 
    {
	if (_p == NULL)
	    return false;
        uint64_t _t = _p->get_tag();
        uint64_t _p2p_partner = _p->get_p2p_partner();
        uint64_t _c = _p->get_communicator();
        return  match( _t, _p2p_partner, _c );
    }   
    
    
    
    
    inline
    uint64_t
    get_tag() const 
    {
        return tag;
    }
    
    inline
    uint64_t
    get_communicator() const 
    {
        return communicator;
    }  
    
    inline
    uint64_t
    get_p2p_partner() const 
    {
        return p2p_partner;
    }  
      
    
    
    inline 
    void
    set_communication_partner( P2PTraceEvent* _e)
    {
        com_partner = _e;
    }
    
    inline 
    P2PTraceEvent * 
    get_communication_partner( ) const
    {
        return com_partner;
    }
       
    
};



#endif // TRACE_EVENT_H

