/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef SERVICE_H
#define SERVICE_H

#include <inttypes.h>
#include "Cube.h"


uint64_t location_extractor(uint64_t _l);


class TraceEvent;
class GeneralEvaluation;  
class TraceEventPoint;
class TraceEventSection;
class TraceEventStrippedSection;

namespace bladeservices
{

enum TraceLoadingProgressStep { TraceReading = 0, P2PMatching = 1, P2PGrouping = 2 };    
    
    
    
inline
uint64_t 
master_id( uint64_t _location)
{
    return _location & 0x00000000fffffffful;
}

inline
uint64_t 
thread_id( uint64_t _location)
{
    return (_location & 0xffffffff00000000ull) >> 32;
}
  
  
inline
uint64_t 
otf2_location_id( cube::Location* _l)
{
    uint32_t thread_id = _l->get_id();
    uint32_t rank = _l->get_parent()->get_rank();
    
    uint64_t id= thread_id;
    id = id << 32 & rank;
    return id;
}
    
inline
cube::Location*
get_cube_location( uint64_t _location, cube::Cube* _cube)
{
    uint64_t _thread = thread_id(_location);
    uint64_t _rank = master_id(_location);
    
    cube::LocationGroup * _lg = _cube->get_location_groupv()[_rank];
    cube::Location* _loc = _lg->get_child(_thread);
    
    return _loc;
}
  
inline
cube::Cnode*
get_cube_callpath( uint64_t _cp, cube::Cube* _cube)
{
    cube::Cnode * _c =(_cp < _cube->get_cnodev().size())?_cube->get_cnodev()[ _cp ]:NULL;
    return _c;
}

inline
std::map<uint64_t, std::map<uint64_t, uint64_t> > 
plain_location_to_tree( std::vector<uint64_t>& _locs)
{
    std::map<uint64_t, std::map<uint64_t, uint64_t> > to_return;
    
    for (std::vector<uint64_t>::iterator iter = _locs.begin(); iter != _locs.end(); ++iter)
    {
        uint64_t  _location = *iter;
        uint64_t _thread = thread_id(_location);
        uint64_t _rank = master_id(_location);
        
        std::map<uint64_t, uint64_t>& _threads_pool = to_return[_rank];
        _threads_pool[_thread] = _location;
    }
    return to_return;
}




inline
std::string
convertToUnit(uint16_t _power)
{
    switch (_power)
    {
        default:
        case 0:
            return "";
        case 3:
            return "m";
        case 6:
            return "mu";
        case 9:
            return "n";
        case 12:
            return "p";
        case 15:
            return "f";
        case 18:
            return "a";            
    }
}





template <typename T1>
bool 
compare_locations( T1 _l1, T1 _l2)
{
    uint64_t _r1 = master_id(location_extractor(_l1));
    uint64_t _r2 = master_id(location_extractor(_l2));
    uint64_t _t1 = thread_id(location_extractor(_l1));
    uint64_t _t2 = thread_id(location_extractor(_l2));
    if (_r1 < _r2)
        return true;
    
    if (_r1 > _r2)
        return false;
    
    if (_t1 < _t2)
        return true;
    
    if (_t1 >= _t2)
        return false;
    
    return false;
}



std::vector<TraceEventPoint>
transform_into_sorted_points(const std::vector<TraceEvent*> & _events);

std::vector<TraceEventSection>
transform_into_sections(const std::vector<TraceEventPoint> & _points, uint64_t, uint64_t);


std::vector<TraceEventStrippedSection> 
evaluate_sections(std::vector<TraceEventSection>&_sections, GeneralEvaluation * metric , uint64_t, uint64_t);


void
adjust_section(TraceEventStrippedSection & _section, uint64_t z_start, uint64_t z_end);

};
#endif // TESTPLUGIN1_H
