/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef SUBTRACE_WIDGET_H
#define SUBTRACE_WIDGET_H
#include <QtGui>
#include <QLabel>
#include <QWidget>
#include <QGraphicsView>
#include <QGraphicsScene>

#include "Trace.h"
#include "CallPathFilter.h"
#include "FlatFilter.h"
#include "SelectionFilter.h"
#include "ZoomFilter.h"
#include "RoleFilter.h"
#include "RegionPropertiesFilter.h"
#include "ResolutionFilter.h"
#include "LocationFilter.h"
#include "ColorMap.h"
#include "GraphicsView.h"


#include "MetricTraceWidget.h"


#include "PluginServices.h"
extern cubepluginapi::PluginServices* blade_service;
extern SelectionFilter * s_filter;
extern ZoomFilter * z_filter;
extern TraceEventFilter  * cf_filter;
extern RoleFilter * role_filter;
extern RegionPropertiesFilter * rp_filter;
extern Trace * trace;
    
class SubTraceWidget : public QWidget
{
    Q_OBJECT
private:

    ResolutionFilter * r_filter;
    LocationFilter * l_filter;
    
    
    uint64_t location;
    QGraphicsScene * scene;
    GraphicsView * view;
    std::vector<TraceEvent * > trace_view;

    ColorPalette * coloring;
    
    
    MetricTraceWidget * metric_trace_widget;
    
    
public slots:
    void 
    changeZoom();
    
    void
    zoomingTip();
    
    void
    updateScene(); 
    
    void
    updateCursor(QPointF);
    
    void
    compactToggle( bool );
    
    
public:
    SubTraceWidget(uint64_t,  QWidget * parent = NULL);
    ~SubTraceWidget();
    
    void 
    resizeEvent(QResizeEvent *event);
    
    QPointF
    getCursor();
    
    void
    setCursor( QPointF&);
    
    
    MetricTraceWidget *
    getMetricTraceWidget()
    {
        return metric_trace_widget;
    }
    
    
    
signals:
    void
    cursorMoved(QPointF);

    
};

#endif // TESTPLUGIN1_H
