/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef TRACE_EVENT_FILTER_H
#define TRACE_EVENT_FILTER_H

#include <string>
#include <vector>
#include <inttypes.h>

#include <QObject>
#include <QPointF>

#include "TraceEvent.h"


typedef enum { FILTER_OK=0, FILTER_FAIL=1, FILTER_CHILDREN=2, FILTER_NO_CHILDREN=3, FILTER_DIRECT_CHILDREN=4} FilterResult;

class TraceEventFilter : public QObject
{

    Q_OBJECT
    
    TraceEventFilter * parent;
    std::vector<TraceEventFilter *> children; 


signals:
    void filterChanged();
    
    void filterChanged(QPointF);
    
public:
    TraceEventFilter()
    {
        active = true;
    }
    
    virtual
    ~TraceEventFilter()
    {
        for (std::vector<TraceEventFilter *>::iterator iter = children.begin(); iter != children.end(); ++iter)
            delete *iter;
    }
    
    
    virtual
    FilterResult 
    apply( TraceEvent * ) = 0;
    
    
    inline
    void
    set_parent( TraceEventFilter * _t)
    {
        parent = _t;
    }
    
    virtual
    void
    add_as_child( TraceEventFilter * _c)
    {
        children.push_back(_c);
        _c->set_parent( this );
        refresh();
    }
    
    inline 
    size_t 
    get_num_children()
    {
        return children.size();
    }
    
    inline 
    TraceEventFilter*
    get_child(size_t i)
    {
        return (i>children.size())?NULL: children[i];
    }
    
    
    virtual
    void
    refresh()
    {
        
    };
    
    virtual
    void
    print() {};
    
    inline
    bool
    is_active()
    {
        return active;
    }
    
    inline 
    void
    set_active( bool _a)
    {
        active = _a;
    }
    
    
protected:
    bool active;
     
};

#endif // FILTER_H

