/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef TRACE_EVENT_GROUP_H
#define TRACE_EVENT_GROUP_H

#include <vector>
#include <list>
#include "TraceEvent.h"




class TraceEventGroup 
{
protected:
    uint32_t number_of_events;
    std::list<TraceEvent*> events;
    std::vector<TraceEvent*> events_as_vector;
    
    TraceEventGroup * prev_group;
    TraceEventGroup * next_group;
    
public:
    TraceEventGroup() 
    {
        events.clear();
        prev_group = NULL;
        next_group = NULL;
    }; 
    
    inline 
    void add_event( TraceEvent * _e)
    {
        events.push_back(_e);
    }
    
    inline
    void delete_event( TraceEvent * _e)
    {
        events.remove(_e);
	
    }
    
    
    
    inline 
    void
    set_prev_group( TraceEventGroup * _p)
    {
        prev_group = _p;
    }
     
    inline 
    void
    set_next_group( TraceEventGroup * _n)
    {
        next_group = _n;
    }
  
    inline 
    TraceEventGroup *
    get_prev_group( ) const
    {
        return prev_group;
    }
     
    inline 
    TraceEventGroup *
    get_next_group( ) const 
    {
        return next_group;
    }
  
  
  
  
  
    inline
    bool
    is_complete()
    {
        return events.size() == number_of_events;
    }
    
    
    inline
    bool
    is_event_for_location( uint64_t _l)
    {
        for (std::list<TraceEvent*>::iterator iter = events.begin(); iter != events.end(); ++iter)
        {
            TraceEvent * _e = (*iter);
            if ((*_e).get_location() == _l)
                return true;
        }
        return false;
    }
    
    inline
    std::vector<TraceEvent*>& 
    get_events()
    {
	if (events.size() != events_as_vector.size())
	{
	    events_as_vector.clear();
	    for (std::list<TraceEvent*>::iterator iter = events.begin(); iter != events.end(); ++iter)
	    {
		TraceEvent * _e = (*iter);
		events_as_vector.push_back( _e);
	    }
	}
        return events_as_vector;
    }
    
    
    
    
};


#endif // TRACE_EVENT_GROUP_H
