/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef TRACE_EVENT_GROUP_WIDGET_H
#define TRACE_EVENT_GROUP_WIDGET_H
#include <QtGui>
#include <QLabel>
#include <QWidget>
#include <QToolBar>
#include <QTimer>
#include <QThread>

#include "MouseLabel.h"
#include "TraceEvent.h"
#include "TraceEventGroupGeneralAnalyzer.h"
#include "TraceEventGroupWaitAtBarrierAnalyzer.h"
#include "TraceEventGroupBarrierCompletionAnalyzer.h"
#include "TraceEventGroupWaitAtN2NAnalyzer.h"
#include "TraceEventGroupN2NCompletionAnalyzer.h"
#include "TraceEventGroupLateBroadcastAnalyzer.h"
#include "TraceEventGroupEarlyReduceAnalyzer.h"
#include "TraceEventGroupLateSenderAnalyzer.h"
#include "TraceEventGroupLateReceiverAnalyzer.h"
#include "TraceGroupGraphicsView.h"
#include "ZoomFilter.h"
extern ZoomFilter * z_filter;

class TraceEventGroupWidget : public QWidget
{
    Q_OBJECT
    
   
private:
    TraceGroupGraphicsView * view;
    QGraphicsScene * scene;
    TraceEventGroupGeneralAnalyzer * ga;
    TraceEventGroupWaitAtBarrierAnalyzer * wait_at_barrier_a;
    TraceEventGroupBarrierCompletionAnalyzer *  barrier_completion_a;
    TraceEventGroupWaitAtN2NAnalyzer * wait_at_n2n_a;
    TraceEventGroupN2NCompletionAnalyzer *  n2n_completion_a;
    TraceEventGroupLateBroadcastAnalyzer * late_broadcast_a;
    TraceEventGroupEarlyReduceAnalyzer * early_reduce_a;
    TraceEventGroupLateSenderAnalyzer * late_sender_a;
    TraceEventGroupLateReceiverAnalyzer * late_receiver_a;
    
    QToolBar * paging_control;
    QAction * prev_group_button;
    QAction * next_group_button; 
    
    
    
    QLabel * description_label;
    std::vector<MouseLabel *> analyzer_labels;
    std::vector<QGraphicsSimpleTextItem*> dynamic_labels;
    std::vector<TraceEventRect*> dynamic_rects;
    std::vector<TraceEvent*> events;
    TraceEventGroup *  current_group;
    
    
protected slots:
        
    virtual
    void
    closeEvent(QCloseEvent * event );
    
    virtual
    void
    hideEvent(QHideEvent * event );
    
    
public slots:
    
    void
    mouseOver(  TraceEventGroupAnalyzer* );
    
    void
    setNextGroup();
    
    void
    setPreviousGroup();
    
public:

    TraceEventGroupWidget( QWidget * parent = NULL);
    ~TraceEventGroupWidget();

    void
    displayEvents(std::vector<TraceEvent*>& events, TraceEventGroup * );

       

};

#endif // TRACE_EVENT_GROUP_WIDGET_H
