/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef TRACE_EVENT_POINT_H
#define TRACE_EVENT_POINT_H

#include <string>
#include <vector>
#include <inttypes.h>
#include <iostream>
#include <stack>
#include <utility>

class Trace;
class TraceEvent;
extern Trace * trace;

class TraceEventPoint 
{
protected:
    
    TraceEvent* te;
    uint64_t timestamp;
    bool start;
    bool including;
    std::vector<uint64_t> metric_values; // simple approach : id -> metric member id, value -> inclusive value of metric
 
    
public:
    TraceEventPoint( );
     
    
    void
    init( TraceEvent*_te, uint64_t _t, bool _s, bool _inc);
    
    inline
    void
    set_timestamp(uint64_t _t)
    {
        timestamp = _t;
    }
    
    
    inline
    void
    set_metric_values(const std::vector<uint64_t>& _p)
    {
        metric_values = _p;
    }
    
    
    inline
    std::vector<uint64_t>&
    get_metric_values( )
    {
        return metric_values;
    }
    
    inline
    size_t
    get_number_of_metrics( ) const
    {
        return metric_values.size();
    }
    

    inline
    uint64_t
    get_metric_value( size_t idx)
    {
        if (idx < metric_values.size())
        {
            return metric_values[idx];
        }
        std::cerr << " No metric with index " << idx << ". Return 0." << std::endl;
        return 0;
    }
    
    
    inline
    bool
    is_start() const
    {
            return start;
    }
    
    inline
    void 
    set_start( bool _s)
    {
        start = _s;
    }
    
    
    inline
    bool
    is_including() const
    {
        return including;
    }
    
    
    inline
    uint64_t 
    get_timestamp() const
    {
        return timestamp;
    }
    

    friend
    std::ostream&
    operator<<( std::ostream&, const TraceEventPoint& );
    
    

    static
    bool
    compareTraceEventPoints( const TraceEventPoint a, const TraceEventPoint b);
    
    
};

    



    

#endif // FILTER_H

