/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef TRACELOADER_H
#define TRACELOADER_H

#include <string>
#include <stack>
#include <map>
#include <vector>

#include "Service.h"

#include "TraceEvent.h"
#include "Trace.h"
#include "RegionClass.h"
#include "LocationClass.h"
#include "LocationGroupClass.h"
#include "SystemTreeNodeClass.h"
#include "MetricMemberClass.h"
#include "MetricClassClass.h"
#include "MetricInstanceClass.h"


#include "MpiCollectiveGroup.h"
#include "MpiCollectiveGroupsCollection.h"
#include "MpiP2PGroup.h"
#include "MpiP2PGroupsCollection.h"
#include "OmpParallelGroup.h"
#include "OmpParallelGroupsCollection.h"


#include "P2PTraceEvent.h"
#include "SendP2PTraceEvent.h"
#include "RecvP2PTraceEvent.h"
#include "ISendP2PTraceEvent.h"
#include "IRecvP2PTraceEvent.h"
#include "WaitTraceEvent.h"
#include "WaitallTraceEvent.h"
#include "BcastTraceEvent.h"
#include "ReduceTraceEvent.h"
#include "ScanTraceEvent.h"
#include "GatherTraceEvent.h"

struct trace_vector
{
    size_t   capacity;
    size_t   size;
    uint64_t members[];    
};


class TraceLoadEnterFilter;
class TraceLoadLeaveFilter;


class TraceLoader 
{
private:
    
  
    
    
    OTF2_Reader* reader;
    OTF2_GlobalDefReader* global_def_reader;
    OTF2_GlobalEvtReader* global_evt_reader ;
    OTF2_GlobalDefReaderCallbacks* global_def_callbacks;
    OTF2_GlobalEvtReaderCallbacks* event_callbacks;
    bool successful_open_def_files;
public:

    TraceLoader( std::string, cube::Cube * );
    ~TraceLoader();
    void
    loadDefinitions();
    
    void
    loadEvents(TraceLoadEnterFilter *, TraceLoadLeaveFilter *, bool );
    
    Trace *
    getTrace();
    
    static
    TraceEvent * 
    createTraceEvent(uint64_t region, uint64_t location);
    
    
    TraceLoadEnterFilter * trace_load_enter_filter;
    TraceLoadLeaveFilter * trace_load_leave_filter;
    Trace * trace;
    
    std::map<uint64_t,std::map<uint64_t, int32_t> > forks;
    
    std::map<uint64_t, std::stack<TraceEvent * > > calls;
    std::map<uint64_t, int32_t> max_levels;
    
    std::string trace_path;
    uint64_t number_of_locations;
    uint64_t number_of_ranks;

    uint64_t timer_resolution;
    uint64_t global_offset;
    uint64_t trace_length;
    
    uint64_t last_read_timestamp;
    uint64_t start_timestamp;
    uint64_t end_timestamp;
    
    std::vector<uint64_t> current_metric_values;
    
    
    
    std::map<uint64_t, LocationClass> locations_classes;  
    std::map<uint64_t, LocationGroupClass> lg_classes;  
    std::map<uint64_t, SystemTreeNodeClass> stn_classes;  
    std::vector<uint64_t> root_stn_classes;  
    

    std::vector< MetricClassClass > metric_classes;
    std::vector< MetricInstanceClass > metric_instances;
    std::vector< MetricMemberClass > metric_members;
    
    std::vector<RegionClass> regions;
    std::vector<CallPathClass> calltree;
    std::map<uint64_t, std::vector<CallPathClass> > calltrees;
    std::vector<std::string> string_defs;
    struct trace_vector* locations;
    
    cube::Cube * cube;
    
    bool callpath_filter_active;
    bool selection_filter_active;
    
    bladeservices::TraceLoadingProgressStep progress_step;
    std::string progress_message;



    MpiCollectiveGroupsCollection * mpi_collective_groups;
    MpiP2PGroupsCollection * mpi_p2p_groups;
    OmpParallelGroupsCollection * omp_parallel_groups;
    
    TraceEvent * dummy_event; // uses to mark a filtered event.
    
    bool grouping;
    
    
    
    double
    get_progress();
    
    std::string&
    get_progress_message();
    
};

#endif // TRACELOADER_H

