/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef ZOOM_FILTER_H
#define ZOOM_FILTER_H

#include <string>
#include <vector>
#include <inttypes.h>
#include <iostream>
#include <stack>
#include <utility>

#include <QPointF>

#include "TraceEventFilter.h"

typedef  std::pair<uint64_t, uint64_t> Point;

class ZoomFilter : public TraceEventFilter 
{
protected:
    
    std::vector<std::pair < uint64_t, uint64_t> > zoom;

    
public:
    
    void
    set_zoom_window( uint64_t, uint64_t);
    
    void
    zoom_in( const QPointF& );
    
    Point
    get_window( QPointF&);
    
    QPointF
    get_window( );
    
     
    void
    go_back();
    
    void
    unzoom();

    void
    move_left();
    
    void
    move_right();
    
    void
    no_zoom();
    
    
    inline
    uint64_t 
    get_zoom_size()
    {
        uint64_t start_window = zoom.back().first;
        uint64_t end_window = zoom.back().second; 
        return end_window  - start_window;
    }
    
    inline
    uint64_t
    get_zoom_start()
    {
        return zoom.back().first;
    }
    
    inline
    uint64_t
    get_zoom_end()
    {
        return zoom.back().second;
    }
    
    inline 
    void
    print()
    {
        uint64_t start_window = zoom.back().first;
        uint64_t end_window = zoom.back().second; 
        std::cout << " Z: " <<  start_window << ":" << end_window << " = " << (get_zoom_size()) << std::endl;
    }
    
    
    uint64_t
    get_time_in_window( double _f);
    
    
    virtual
    void
    refresh();
    
    
    virtual
    FilterResult 
    apply( TraceEvent *  );
    
};

#endif // FILTER_H

