/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

#ifndef BLADESCANNER_H
#define BLADESCANNER_H

// Flex expects the signature of yylex to be defined in the macro YY_DECL, and
// the C++ parser expects it to be declared. We can factor both as follows.

#ifndef YY_DECL

#define YY_DECL                                         \
    bladeparser::BladeParser::token_type                             \
    bladeparser::BladeScanner::lex(                          \
        bladeparser::BladeParser::semantic_type * yylval,             \
        bladeparser::BladeParser::location_type * yylloc              \
        )
#endif

#ifndef __FLEX_LEXER_H
//#define yyFlexLexer Cube3FlexLexer
#include <BladeFlexLexer.h>
#undef yyFlexLexer
#endif

#include "parser.hpp"

namespace bladeparser
{
/** Scanner is a derived class to add some extra function to the scanner
 * class. Flex itself creates a class named yyFlexLexer, which is renamed using
 * macros to CubeparserFlexLexer. However we change the context of the generated
 * yylex() function to be contained within the Scanner class. This is required
 * because the yylex() defined in yyFlexLexer has no parameters. */
class BladeScanner : public yyFlexLexer
{
public:
    /** Create a new scanner object. The streams arg_yyin and arg_yyout default
     * to cin and cout, but that assignment is only made when initializing in
     * yylex(). */
    BladeScanner( std::istream*        arg_yyin = 0,
                    std::ostream*        arg_yyout = 0,
                    BladeParseContext* parseContext = NULL );

    /** Required for virtual functions */
    virtual
    ~BladeScanner();

    /** This is the main lexing function. It is generated by flex according to
     * the macro declaration YY_DECL above. The generated bison parser then
     * calls this virtual function to fetch new tokens. */
    virtual BladeParser::token_type
    lex(
        BladeParser::semantic_type* yylval,
        BladeParser::location_type* yylloc
        );

private:

    BladeParseContext* parseContext;
};
} // namespace parser

#endif // SCANNER_H
